// external lib
import { ListenerRss, ListenerRSSInfos } from "listener-rss";

// local lib
import { ListenerRssAggregator } from "../src/";

// Unit test
import { join as joinPath } from "path";
import events from "events";
import nock from "nock";
import * as chai from "chai";
import sinon from "ts-sinon";
import sinonChai from "sinon-chai";

import { withFile } from "tmp-promise";

chai.use(sinonChai);

const expect = chai.expect;

const dataWithHistory: ListenerRSSInfos[] = require("./RessourcesTest/RealRessources/save.json");
const dataWithoutHistory = dataWithHistory.map((data) => ({
  ...data,
  lastEntriesLinks: [],
}));

describe("test class ManageListener", function () {
  describe("test saving", function () {
    it("should save the inserted objects", async function () {
      await withFile(async ({ path }) => {
        // given
        const ml = new ListenerRssAggregator(
          await ListenerRssAggregator.instantiateAggregator(path)
        );

        await ml.saveOverride(dataWithoutHistory);

        // when
        const ml_bis = new ListenerRssAggregator(
          await ListenerRssAggregator.instantiateAggregator(path)
        );
        // expect
        expect(await ml_bis.getAllConfigs()).to.have.deep.members(
          dataWithoutHistory
        );
      });
    });
  });

  describe("test events", function () {
    beforeEach(function () {
      nock.disableNetConnect();
    });

    afterEach(function () {
      nock.cleanAll();
    });

    it("should call correctly the events (no aggregation)", async function () {
      await withFile(async ({ path }) => {
        // given
        const clock = sinon.useFakeTimers();
        const ml = new ListenerRssAggregator(
          await ListenerRssAggregator.instantiateAggregator(path)
        );
        for (const item of dataWithoutHistory) await ml.registerListener(item);

        const updateSpy = sinon.spy();
        const newEntriesSpy = sinon.spy();

        const tabChannelId = [
          "UCOuIgj0CYCXCvjWywjDbauw",
          "UCh2YBKhYIy-_LtfCIn2Jycg",
          "UCY7klexd1qEqxgqYK6W7BVQ",
        ];

        ml.on("update", updateSpy);
        ml.on("newEntries", newEntriesSpy);
        tabChannelId.forEach((item: string) => {
          nock("https://www.youtube.com")
            .get(`/feeds/videos.xml?channel_id=${item}`)
            .replyWithFile(
              200,
              joinPath(__dirname, `RessourcesTest/RealRessources/${item}.rss`),
              { "content-type": "text/xml", charset: "utf-8" }
            )
            .persist();
        });
        ml.startAll();

        // here we're testing the first call
        // when
        await events.once(ml, "update");
        await events.once(ml, "update");
        await events.once(ml, "update");
        //expect
        expect(updateSpy).to.have.been.calledThrice;
        expect(newEntriesSpy).to.have.been.calledThrice;

        // here we're testing the second call for the first listener
        // given
        updateSpy.resetHistory();
        newEntriesSpy.resetHistory();

        // when
        await clock.tickAsync(10000);
        await events.once(ml, "update");

        //expect
        expect(updateSpy).to.have.been.calledOnce;
        expect(newEntriesSpy).to.not.have.been.called;

        // here we're testing the thrid call for the first listener and second for the second listener
        // given
        updateSpy.resetHistory();

        // when
        await clock.tickAsync(10000);
        await events.once(ml, "update");
        await events.once(ml, "update");

        //expect
        expect(updateSpy).to.have.been.calledTwice;
        expect(newEntriesSpy).to.not.have.been.called;

        // here we're testing the fourth call for the first listener and second for the third listener
        // given
        updateSpy.resetHistory();

        // when
        await clock.tickAsync(10000);
        await events.once(ml, "update");
        await events.once(ml, "update");

        //expect
        expect(updateSpy).to.have.been.calledTwice;
        expect(newEntriesSpy).to.not.have.been.called;

        ml.stopAll();
      });
    });

    it("should call correctly the events with a new entry (no aggregation)", async function () {
      await withFile(async ({ path }) => {
        // given
        const clock = sinon.useFakeTimers();
        const ml = new ListenerRssAggregator(
          await ListenerRssAggregator.instantiateAggregator(path)
        );
        for (const item of dataWithHistory) await ml.registerListener(item);

        const updateSpy = sinon.spy();
        const newEntriesSpy = sinon.spy();

        const tabChannelId = [
          "UCOuIgj0CYCXCvjWywjDbauw",
          "UCh2YBKhYIy-_LtfCIn2Jycg",
          "UCY7klexd1qEqxgqYK6W7BVQ",
        ];

        ml.on("update", updateSpy);
        ml.on("newEntries", newEntriesSpy);
        tabChannelId.forEach((item: string) => {
          nock("https://www.youtube.com")
            .get(`/feeds/videos.xml?channel_id=${item}`)
            .once()
            .replyWithFile(
              200,
              joinPath(__dirname, `RessourcesTest/RealRessources/${item}.rss`),
              { "content-type": "text/xml", charset: "utf-8" }
            );
        });
        ml.startAll();

        // here we're testing the first call
        // when
        await events.once(ml, "update");
        await events.once(ml, "update");
        await events.once(ml, "update");

        //expect
        expect(updateSpy).to.have.been.calledThrice;
        expect(newEntriesSpy).to.not.have.been.called;

        // here we're testing the second call for the first listener
        // given
        updateSpy.resetHistory();
        newEntriesSpy.resetHistory();

        tabChannelId.forEach((item: string) => {
          nock("https://www.youtube.com")
            .get(`/feeds/videos.xml?channel_id=${item}`)
            .replyWithFile(
              200,
              joinPath(
                __dirname,
                `RessourcesTest/RealRessources/WithUpdate/${item}.rss`
              ),
              { "content-type": "text/xml", charset: "utf-8" }
            )
            .persist();
        });

        // when
        await clock.tickAsync(10000);
        await events.once(ml, "update");

        //expect
        expect(updateSpy).to.have.been.calledOnce;
        expect(newEntriesSpy).to.have.been.calledOnce;

        // here we're testing the thrid call for the first listener and second for the second listener
        // given
        updateSpy.resetHistory();
        newEntriesSpy.resetHistory();

        // when
        await clock.tickAsync(10000);
        await events.once(ml, "update");
        await events.once(ml, "update");

        //expect
        expect(updateSpy).to.have.been.calledTwice;
        expect(newEntriesSpy).to.have.been.calledOnce;

        // here we're testing the fourth call for the first listener and second for the third listener
        // given
        updateSpy.resetHistory();
        newEntriesSpy.resetHistory();

        // when
        await clock.tickAsync(10000);
        await events.once(ml, "update");
        await events.once(ml, "update");

        //expect
        expect(updateSpy).to.have.been.calledTwice;
        expect(newEntriesSpy).to.have.been.calledOnce;

        ml.stopAll();
      });
    });
  });
});
